@@ -1,6 +1,6 @@
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Copyright (C) 1992-1998 Washington University School of Medicine
+ * Copyright (C) 1992-2003 Washington University School of Medicine
  * All Rights Reserved
  * 
  *     This source code is distributed under the terms of the
@@ -14,25 +14,21 @@
  * Printing out or naming various useful things from HMMER
  * innards.
  * 
- * RCS $Id: debug.c,v 1.9 1998/10/09 17:40:25 eddy Exp $
+ * CVS $Id: debug.c,v 1.13 2003/04/14 16:00:16 eddy Exp $
  */
 
+#include "config.h"
+#include "squidconf.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <ctype.h>
 
 #include "structs.h"
-#include "config.h"
 #include "funcs.h" 
 #include "squid.h"
 
-#ifdef MEMDEBUG
-#include "dbmalloc.h"
-#endif
-
-
-
 /* Function: Statetype()
  * 
  * Purpose:  Returns the state type in text.
@@ -56,6 +52,29 @@ Statetype(char st)
   }
 }
 
+/* Function: AlphabetType2String()
+ * Date:     SRE, Sun Dec 24 11:33:40 2000 [St. Louis]
+ *
+ * Purpose:  Returns a string "protein" for hmmAMINO,
+ *           "nucleic acid" for hmmNUCLEIC, etc... used 
+ *           for formatting diagnostics.
+ *
+ * Args:     type - Alphabet type, e.g. hmmAMINO
+ *
+ * Returns:  char *
+ */
+char *
+AlphabetType2String(int type)
+{
+  switch (type) {
+  case hmmAMINO:     return "protein";
+  case hmmNUCLEIC:   return "nucleic acid";
+  case hmmNOTSETYET: return "unknown";
+  default:           return "BOGUS";
+  }
+}
+
+
 /* Function: P7PrintTrace()
  * 
  * Purpose:  Print out a traceback structure.
@@ -73,6 +92,11 @@ P7PrintTrace(FILE *fp, struct p7trace_s *tr, struct plan7_s *hmm, char *dsq)
   int sym;
   int sc; 
 
+  if (tr == NULL) {
+    fprintf(fp, " [ trace is NULL ]\n");
+    return;
+  }
+
   if (hmm == NULL) {
     fprintf(fp, "st  node   rpos  - traceback len %d\n", tr->tlen);
     fprintf(fp, "--  ---- ------\n");
@@ -114,14 +138,14 @@ P7PrintTrace(FILE *fp, struct p7trace_s *tr, struct plan7_s *hmm, char *dsq)
 	else if (tr->statetype[tpos] == STI) 
 	  {
 	    fprintf(fp, " %8d %c", hmm->isc[sym][tr->nodeidx[tpos]], 
-		    tolower(Alphabet[sym]));
+		    (char) tolower((int) Alphabet[sym]));
 	    sc += hmm->isc[sym][tr->nodeidx[tpos]];
 	  }
 	else if ((tr->statetype[tpos] == STN && tr->statetype[tpos-1] == STN) ||
 		 (tr->statetype[tpos] == STC && tr->statetype[tpos-1] == STC) ||
 		 (tr->statetype[tpos] == STJ && tr->statetype[tpos-1] == STJ))
 	  {
-	    fprintf(fp, " %8d %c", 0, tolower(Alphabet[sym]));
+	    fprintf(fp, " %8d %c", 0, (char) tolower((int) Alphabet[sym]));
 	  }
       } else {
 	fprintf(fp, " %8s %c", "-", '-');
