@@ -1,6 +1,6 @@
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Copyright (C) 1992-1998 Washington University School of Medicine
+ * Copyright (C) 1992-2003 Washington University School of Medicine
  * All Rights Reserved
  * 
  *     This source code is distributed under the terms of the
@@ -9,7 +9,7 @@
  ************************************************************/
 
 /* funcs.h 
- * RCS $Id: funcs.h,v 1.27 1998/12/09 17:41:56 eddy Exp $
+ * RCS $Id: funcs.h,v 1.36 2003/04/14 16:00:16 eddy Exp $
  *
  * Declarations of external functions in HMMER.
  */            
@@ -20,6 +20,7 @@
 #include "config.h"
 #include "structs.h"
 #include "squid.h"
+#include "msa.h"
 
 /* alphabet.c
  * Configuration of global alphabet information
@@ -29,7 +30,7 @@ extern void  SetAlphabet(int type);
 extern int   SymbolIndex(char sym);
 extern char *DigitizeSequence(char *seq, int L);
 extern char *DedigitizeSequence(char *dsq, int L);
-extern void  DigitizeAlignment(char **aseqs, AINFO *ainfo, char ***ret_dsqs);
+extern void  DigitizeAlignment(MSA *msa, char ***ret_dsqs);
 extern void  P7CountSymbol(float *counters, char sym, float wt);
 extern void  DefaultGeneticCode(int *aacode);
 extern void  DefaultCodonBias(float *codebias);
@@ -37,8 +38,11 @@ extern void  DefaultCodonBias(float *codebias);
 /* from core_algorithms.c
  * Clean research/demonstration versions of basic algorithms.
  */
-extern struct dpmatrix_s *AllocPlan7Matrix(int rows, int M, int ***xmx, 
-					   int ***mmx, int ***imx, int ***dmx);
+extern struct dpmatrix_s *CreatePlan7Matrix(int N, int M, int padN, int padM);
+extern void   ResizePlan7Matrix(struct dpmatrix_s *mx, int N, int M, 
+				int ***xmx, int ***mmx, int ***imx, int ***dmx);
+struct dpmatrix_s *AllocPlan7Matrix(int rows, int M, 
+				    int ***xmx, int ***mmx, int ***imx, int ***dmx);
 extern struct dpshadow_s *AllocShadowMatrix(int rows, int M, char ***xtb, 
 					    char ***mtb, char ***itb, char ***dtb);
 extern void  FreePlan7Matrix(struct dpmatrix_s *mx);
@@ -48,32 +52,48 @@ extern int   P7SmallViterbiSize(int L, int M);
 extern int   P7WeeViterbiSize(int L, int M);
 extern float P7Forward(char *dsq, int L, struct plan7_s *hmm, 
 			  struct dpmatrix_s **ret_mx);
-extern float P7Viterbi(char *dsq, int L, struct plan7_s *hmm, 
+extern float P7Viterbi(char *dsq, int L, struct plan7_s *hmm, struct dpmatrix_s *mx,
 			  struct p7trace_s **ret_tr);
 extern void  P7ViterbiTrace(struct plan7_s *hmm, char *dsq, int L,
 			   struct dpmatrix_s *mx, struct p7trace_s **ret_tr);
-extern float P7SmallViterbi(char *dsq, int L, struct plan7_s *hmm, struct p7trace_s **ret_tr);
+extern float P7SmallViterbi(char *dsq, int L, struct plan7_s *hmm, struct dpmatrix_s *mx, struct p7trace_s **ret_tr);
 extern float P7ParsingViterbi(char *dsq, int L, struct plan7_s *hmm, 
 			      struct p7trace_s **ret_tr);
 extern float P7WeeViterbi(char *dsq, int L, struct plan7_s *hmm, 
 			  struct p7trace_s **ret_tr);
 extern float Plan7ESTViterbi(char *dsq, int L, struct plan7_s *hmm, 
 			     struct dpmatrix_s **ret_mx);
-extern struct p7trace_s *P7ViterbiAlignAlignment(char **aseq, AINFO *ainfo, struct plan7_s *hmm);
+extern struct p7trace_s *P7ViterbiAlignAlignment(MSA *msa, struct plan7_s *hmm);
 extern struct p7trace_s *ShadowTrace(struct dpshadow_s *tb, struct plan7_s *hmm, int L);
-
+extern float  PostprocessSignificantHit(struct tophit_s *ghit, struct tophit_s *dhit, struct p7trace_s   *tr, struct plan7_s *hmm, char *dsq, int L, char *seqname, char *seqacc, char *seqdesc, int do_forward, float sc_override, int do_null2, struct threshold_s *thresh, int hmmpfam_mode);
 
 
 /* from debug.c
  * Debugging output of various sorts.
  */
 extern char *Statetype(char st);
+extern char *AlphabetType2String(int type);
 extern void P7PrintTrace(FILE *fp, struct p7trace_s *tr, 
 			 struct plan7_s *hmm, char *dsq);
 extern void P7PrintPrior(FILE *fp, struct p7prior_s *pri);
 extern int  TraceCompare(struct p7trace_s *t1, struct p7trace_s *t2);
 extern int  TraceVerify(struct p7trace_s *tr, int M, int N);
 
+/* 
+ * from display.c
+ * Ian Holmes' functions for displaying HMMER2 data structures, especially
+ * for posterior probabilities in alignments.
+ */
+extern void DisplayPlan7Matrix(char *dsq, int L, struct plan7_s *hmm,
+			       struct dpmatrix_s *mx);
+extern void DisplayPlan7Posteriors(int L, struct plan7_s *hmm,
+				   struct dpmatrix_s *forward, struct dpmatrix_s *backward,
+				   struct p7trace_s *viterbi, struct p7trace_s *optacc);
+extern void DisplayPlan7PostAlign(int L, struct plan7_s *hmm,
+				  struct dpmatrix_s *forward, struct dpmatrix_s *backward,
+				  struct p7trace_s **alignment, int A);
+
+
 /* from emit.c
  * Generation of sequences/traces from an HMM
  */
@@ -129,7 +149,6 @@ extern int      HMMFileRead(HMMFILE *hmmfp, struct plan7_s **ret_hmm);
 extern void     HMMFileClose(HMMFILE *hmmfp);
 extern int      HMMFileFormat(HMMFILE *hmmfp);
 extern void     HMMFileRewind(HMMFILE *hmmfp);
-extern void     HMMFilePositionByOffset(HMMFILE *hmmfp, long offset);
 extern int      HMMFilePositionByName(HMMFILE *hmmfp, char *name);
 extern int      HMMFilePositionByIndex(HMMFILE *hmmfp, int idx);
 extern void     WriteAscHMM(FILE *fp, struct plan7_s *hmm);
@@ -160,21 +179,20 @@ extern float P_PvecGivenDirichlet(float *p, int n, float *alpha);
 /* from misc.c
  * Miscellaneous functions with no home
  */
-extern void  Banner(FILE *fp, char *banner);
+extern void  HMMERBanner(FILE *fp, char *banner);
 extern char *Getword(FILE *fp, int type); 
 extern char *Getline(char *s, int n, FILE *fp);
-
+extern int   SetAutocuts(struct threshold_s *thresh, struct plan7_s *hmm);
 
 /* from modelmakers.c
  * Model construction algorithms
  */
-extern void P7Handmodelmaker(char **aseq, char **dsq, AINFO *ainfo,
-			     struct plan7_s **ret_hmm,
+extern void P7Handmodelmaker(MSA *msa, char **dsq, struct plan7_s **ret_hmm,
 			     struct p7trace_s ***ret_tr);
-extern void P7Fastmodelmaker(char **aseq, char **dsq, AINFO *ainfo,
+extern void P7Fastmodelmaker(MSA *msa, char **dsq, 
 			     float maxgap, struct plan7_s **ret_hmm, 
 			     struct p7trace_s ***ret_tr);
-extern void P7Maxmodelmaker(char **aseqs, char **dsq, AINFO *ainfo,
+extern void P7Maxmodelmaker(MSA *msa, char **dsq, 
 			    float maxgap, struct p7prior_s *prior, 
 			    float *null, float null_p1, float mpri, 
 			    struct plan7_s **ret_hmm,
@@ -217,6 +235,20 @@ extern int  P9FreeHMM(struct plan9_s *hmm);
 extern void P9Renormalize(struct plan9_s *hmm);
 extern void P9DefaultNullModel(float *null);
 
+/* 
+ * from postprob.c
+ * Functions for working with posterior probabilities within alignments
+ */
+extern float P7OptimalAccuracy(char *dsq, int L, struct plan7_s *hmm, struct p7trace_s **ret_tr);
+extern float P7Backward(char *dsq, int L, struct plan7_s *hmm, 	struct dpmatrix_s **ret_mx);
+extern void  P7EmitterPosterior(int L, struct plan7_s *hmm, struct dpmatrix_s *forward,
+				struct dpmatrix_s *backward, struct dpmatrix_s *mx);
+extern float P7FillOptimalAccuracy(int L, int M, struct dpmatrix_s *posterior,
+				   struct dpmatrix_s *mx, struct p7trace_s **ret_tr);
+extern void  P7OptimalAccuracyTrace(int L, int M, struct dpmatrix_s *posterior,
+				    struct dpmatrix_s *mx, struct p7trace_s **ret_tr);
+extern char *PostalCode(int L, struct dpmatrix_s *mx, struct p7trace_s *tr);
+
 /* from prior.c
  * Dirichlet priors
  */
@@ -229,7 +261,8 @@ extern void PAMPrior(char *pamfile, struct p7prior_s *pri, float pamwgt);
 extern void P7DefaultNullModel(float *null, float *ret_p1);
 extern void P7ReadNullModel(char *rndfile, float *null, float *ret_p1);
 extern void P7PriorifyHMM(struct plan7_s *hmm, struct p7prior_s *pri);
-extern void P7PriorifyTransitionVector(float *t, struct p7prior_s *prior);
+extern void P7PriorifyTransitionVector(float *t, struct p7prior_s *prior, 
+				       float tq[MAXDCHLET]);
 extern void P7PriorifyEmissionVector(float *vec, struct p7prior_s *pri, 
 				     int num, float eq[MAXDCHLET], 
 				     float e[MAXDCHLET][MAXABET],
@@ -271,7 +304,7 @@ extern void   FreeFancyAli(struct fancyali_s *ali);
 extern void   RegisterHit(struct tophit_s *h, double sortkey, 
 			  double pvalue, float score, 
 			  double motherp, float mothersc,
-			  char *name, char *desc, 
+			  char *name, char *acc, char *desc, 
 			  int sqfrom, int sqto, int sqlen, 
 			  int hmmfrom, int hmmto, int hmmlen, 
 			  int domidx, int ndom, 
@@ -279,7 +312,7 @@ extern void   RegisterHit(struct tophit_s *h, double sortkey,
 extern void GetRankedHit(struct tophit_s *h, int rank, 
 			 double *r_pvalue, float *r_score, 
 			 double *r_motherp, float *r_mothersc,
-			 char **r_name, char **r_desc,
+			 char **r_name, char **r_acc, char **r_desc,
 			 int *r_sqfrom, int *r_sqto, int *r_sqlen,
 			 int *r_hmmfrom, int *r_hmmto, int *r_hmmlen,
 			 int *r_domidx, int *r_ndom,
@@ -300,9 +333,9 @@ extern void  P7ReverseTrace(struct p7trace_s *tr);
 extern void  P7TraceCount(struct plan7_s *hmm, char *dsq, float wt, 
 			  struct p7trace_s *tr);
 extern float P7TraceScore(struct plan7_s *hmm, char *dsq, struct p7trace_s *tr);
-extern void  P7Traces2Alignment(char **dsq, SQINFO *sqinfo, float *wgt, int nseq, int M, 
-				struct p7trace_s **tr, int matchonly, 
-				char ***ret_aseqs, AINFO *ainfo);
+extern MSA  *P7Traces2Alignment(char **dsq, SQINFO *sqinfo, float *wgt, 
+				int nseq, int M, 
+				struct p7trace_s **tr, int matchonly);
 extern int  TransitionScoreLookup(struct plan7_s *hmm, char st1, 
 				  int k1, char st2, int k2);
 extern struct fancyali_s *CreateFancyAli(struct p7trace_s *tr, struct plan7_s *hmm,
