@@ -1,6 +1,6 @@
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Copyright (C) 1992-1998 Washington University School of Medicine
+ * Copyright (C) 1992-2003 Washington University School of Medicine
  * All Rights Reserved
  * 
  *     This source code is distributed under the terms of the
@@ -12,21 +12,21 @@
  * SRE, Thu Dec 18 16:05:29 1997 [St. Louis]
  * 
  * main() for aligning a set of sequences to an HMM.
- * RCS $Id: hmmalign.c,v 1.7 1998/10/13 21:51:14 eddy Exp $
+ * CVS $Id: hmmalign.c,v 1.14 2003/04/14 16:00:16 eddy Exp $
  */ 
 
+#include "config.h"		/* compile-time configuration constants */
+#include "squidconf.h"
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "structs.h"		/* data structures, macros, #define's   */
-#include "config.h"		/* compile-time configuration constants */
 #include "funcs.h"		/* function declarations                */
 #include "globals.h"		/* alphabet global variables            */
 #include "squid.h"		/* general sequence analysis library    */
-
-#ifdef MEMDEBUG
-#include "dbmalloc.h"
-#endif
+#include "msa.h"		/* squid's multiple alignment i/o       */
 
 static char banner[] = "hmmalign - align sequences to an HMM profile";
 
@@ -40,6 +40,7 @@ Available options are:\n\
 ";
 
 static char experts[] = "\
+   --informat <s>: sequence file is in format <s>\n\
    --mapali <f>  : include alignment in file <f> using map in HMM\n\
    --withali <f> : include alignment to (fixed) alignment in file <f>\n\
 \n";
@@ -49,6 +50,7 @@ static struct opt_s OPTIONS[] = {
   { "-m", TRUE, sqdARG_NONE   } ,
   { "-o", TRUE, sqdARG_STRING },
   { "-q", TRUE, sqdARG_NONE   },
+  { "--informat",FALSE, sqdARG_STRING },
   { "--mapali",  FALSE, sqdARG_STRING },
   { "--withali", FALSE, sqdARG_STRING },
 };
@@ -63,17 +65,17 @@ main(int argc, char **argv)
 {
   char            *hmmfile;	/* file to read HMMs from                  */
   HMMFILE         *hmmfp;       /* opened hmmfile for reading              */
+  struct plan7_s  *hmm;         /* HMM to align to                         */ 
   char            *seqfile;     /* file to read target sequence from       */ 
   int              format;	/* format of seqfile                       */
   char           **rseq;        /* raw, unaligned sequences                */ 
   SQINFO          *sqinfo;      /* info associated with sequences          */
   char           **dsq;         /* digitized raw sequences                 */
   int              nseq;        /* number of sequences                     */  
-  char           **aseq;        /* aligned sequences                       */
-  AINFO            ainfo;       /* alignment information                   */
-  float           *wgt;         /* per-sequence weights                    */
+  float           *wgt;		/* weights to assign to alignment          */
+  MSA             *msa;         /* alignment that's created                */    
   int              i;
-  struct plan7_s    *hmm;       /* HMM to align to                         */ 
+  struct dpmatrix_s *mx;        /* growable DP matrix                      */
   struct p7trace_s **tr;        /* traces for aligned sequences            */
 
   char *optname;                /* name of option found by Getopt()         */
@@ -86,16 +88,11 @@ main(int argc, char **argv)
   char *withali;                /* name of additional alignment file to align */
   char *mapali;                 /* name of additional alignment file to map   */
   
-#ifdef MEMDEBUG
-  unsigned long histid1, histid2, orig_size, current_size;
-  orig_size = malloc_inuse(&histid1);
-  fprintf(stderr, "[... memory debugging is ON ...]\n");
-#endif
-
   /*********************************************** 
    * Parse command line
    ***********************************************/
   
+  format    = SQFILE_UNKNOWN;	/* default: autodetect format */
   matchonly = FALSE;
   outfile   = NULL;
   be_quiet  = FALSE;
@@ -109,12 +106,17 @@ main(int argc, char **argv)
     else if (strcmp(optname, "-q")        == 0) be_quiet = TRUE; 
     else if (strcmp(optname, "--mapali")  == 0) mapali   = optarg;
     else if (strcmp(optname, "--withali") == 0) withali  = optarg;
+    else if (strcmp(optname, "--informat") == 0) {
+	format = String2SeqfileFormat(optarg);
+	if (format == SQFILE_UNKNOWN) 
+	  Die("unrecognized sequence file format \"%s\"", optarg);
+      }
     else if (strcmp(optname, "-h") == 0) 
       {
-	Banner(stdout, banner);
+	HMMERBanner(stdout, banner);
 	puts(usage);
 	puts(experts);
-	exit(0);
+	exit(EXIT_SUCCESS);
       }
   }
   if (argc - optind != 2)
@@ -123,6 +125,13 @@ main(int argc, char **argv)
   hmmfile = argv[optind++];
   seqfile = argv[optind++]; 
 
+  /* Try to work around inability to autodetect from a pipe or .gz:
+   * assume FASTA format
+   */
+  if (format == SQFILE_UNKNOWN &&
+      (Strparse("^.*\\.gz$", seqfile, 0) || strcmp(seqfile, "-") == 0))
+    format = SQFILE_FASTA;
+
  /*********************************************** 
   * Open HMM file (might be in HMMERDB or current directory).
   * Read a single HMM from it.
@@ -153,14 +162,6 @@ main(int argc, char **argv)
    * Read all seqs from it.
    ***********************************************/
 
-  if (! SeqfileFormat(seqfile, &format, NULL))
-    switch (squid_errno) {
-    case SQERR_NOFILE: 
-      Die("Sequence file %s could not be opened for reading", seqfile); /*FALLTHRU*/
-    case SQERR_FORMAT: 
-    default:           
-      Die("Failed to determine format of sequence file %s", seqfile);
-    }
   if (! ReadMultipleRseqs(seqfile, format, &rseq, &sqinfo, &nseq))
     Die("Failed to read any sequences from file %s", seqfile);
 
@@ -170,7 +171,7 @@ main(int argc, char **argv)
 
   if (! be_quiet) 
     {
-      Banner(stdout, banner);
+      HMMERBanner(stdout, banner);
       printf(   "HMM file:             %s\n", hmmfile);
       printf(   "Sequence file:        %s\n", seqfile);
       printf("- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n\n");
@@ -184,6 +185,7 @@ main(int argc, char **argv)
    */
   dsq = MallocOrDie(sizeof(char *) * nseq);
   tr  = MallocOrDie(sizeof(struct p7trace_s *) * nseq);
+  mx  = CreatePlan7Matrix(1, hmm->M, 25, 0);
 
   /* Align each sequence to the model, collect traces
    */
@@ -192,10 +194,11 @@ main(int argc, char **argv)
       dsq[i] = DigitizeSequence(rseq[i], sqinfo[i].len);
 
       if (P7ViterbiSize(sqinfo[i].len, hmm->M) <= RAMLIMIT)
-	(void) P7Viterbi(dsq[i], sqinfo[i].len, hmm, &(tr[i]));
+	(void) P7Viterbi(dsq[i], sqinfo[i].len, hmm, mx, &(tr[i]));
       else
-	(void) P7SmallViterbi(dsq[i], sqinfo[i].len, hmm, &(tr[i]));
+	(void) P7SmallViterbi(dsq[i], sqinfo[i].len, hmm, mx, &(tr[i]));
     }
+  FreePlan7Matrix(mx);
 
   /* Include an aligned alignment, if desired.
    */
@@ -208,8 +211,7 @@ main(int argc, char **argv)
    */ 
   wgt = MallocOrDie(sizeof(float) * nseq);
   FSet(wgt, nseq, 1.0);
-  P7Traces2Alignment(dsq, sqinfo, wgt, nseq, hmm->M, tr, matchonly,
-		     &aseq, &ainfo);
+  msa = P7Traces2Alignment(dsq, sqinfo, wgt, nseq, hmm->M, tr, matchonly);
 
   /*********************************************** 
    * Output the alignment
@@ -217,12 +219,12 @@ main(int argc, char **argv)
   
   if (outfile != NULL && (ofp = fopen(outfile, "w")) != NULL)
     {
-      WriteSELEX(ofp, aseq, &ainfo, 50);
+      WriteStockholm(ofp, msa);
       printf("Alignment saved in file %s\n", outfile);
       fclose(ofp);
     }
   else
-    WriteSELEX(stdout, aseq, &ainfo, 50);
+    WriteStockholm(stdout, msa);
 
   /*********************************************** 
    * Cleanup and exit
@@ -234,7 +236,7 @@ main(int argc, char **argv)
       FreeSequence(rseq[i], &(sqinfo[i]));
       free(dsq[i]);
     }
-  FreeAlignment(aseq, &ainfo);
+  MSAFree(msa);
   FreePlan7(hmm);
   free(sqinfo);
   free(rseq);
@@ -243,12 +245,6 @@ main(int argc, char **argv)
   free(tr);
 
   SqdClean();
-
-#ifdef MEMDEBUG
-  current_size = malloc_inuse(&histid2);
-  if (current_size != orig_size) malloc_list(2, histid1, histid2);
-  else fprintf(stderr, "[No memory leaks.]\n");
-#endif
   return 0;
 }
 
@@ -274,69 +270,67 @@ main(int argc, char **argv)
  * Returns:  new, realloc'ed arrays for rsq, dsq, sqinfo, tr; nseq is
  *           increased to nseq+ainfo.nseq.
  */
-void
+static void
 include_alignment(char *seqfile, struct plan7_s *hmm, int do_mapped,
 		  char ***rsq, char ***dsq, SQINFO **sqinfo, 
 		  struct p7trace_s ***tr, int *nseq)
 {
   int format;			/* format of alignment file */
-  char **aseq;			/* aligned seqs             */
+  MSA   *msa;			/* alignment to align to    */
+  MSAFILE *afp;
+  SQINFO  *newinfo;		/* sqinfo array from msa */
   char **newdsq;
   char **newrseq;
-  AINFO ainfo;			/* info that goes with aseq */
   int   idx;			/* counter over aseqs       */
   struct p7trace_s *master;     /* master trace             */
   struct p7trace_s **addtr;     /* individual traces for aseq */
 
-  if (! SeqfileFormat(seqfile, &format, NULL))
-    switch (squid_errno) {
-    case SQERR_NOFILE: 
-      Die("Alignment file %s could not be opened for reading", seqfile);
-      /*FALLTHRU*/ /* a white lie to shut lint up */
-    case SQERR_FORMAT: 
-    default:           
-      Die("Failed to determine format of alignment file %s", seqfile);
-    }
-				/* read the alignment from file */
-  if (! ReadAlignment(seqfile, format, &aseq, &ainfo))
-    Die("Failed to read aligned sequence file %s", seqfile);
-  for (idx = 0; idx < ainfo.nseq; idx++)
-    s2upper(aseq[idx]);
+  format = MSAFILE_UNKNOWN;	/* invoke Babelfish */
+  if ((afp = MSAFileOpen(seqfile, format, NULL)) == NULL)
+    Die("Alignment file %s could not be opened for reading", seqfile);
+  if ((msa = MSAFileRead(afp)) == NULL)
+    Die("Failed to read an alignment from %s\n", seqfile);
+  MSAFileClose(afp);
+  for (idx = 0; idx < msa->nseq; idx++)
+    s2upper(msa->aseq[idx]);
+  newinfo = MSAToSqinfo(msa);
+
 				/* Verify checksums before mapping */
-  if (do_mapped && GCGMultchecksum(aseq, ainfo.nseq) != hmm->checksum)
+  if (do_mapped && GCGMultchecksum(msa->aseq, msa->nseq) != hmm->checksum)
     Die("The checksums for alignment file %s and the HMM alignment map don't match.", 
 	seqfile);
 				/* Get a master trace */
-  if (do_mapped) master = MasterTraceFromMap(hmm->map, hmm->M, ainfo.alen);
-  else           master = P7ViterbiAlignAlignment(aseq, &ainfo, hmm);
+  if (do_mapped) master = MasterTraceFromMap(hmm->map, hmm->M, msa->alen);
+  else           master = P7ViterbiAlignAlignment(msa, hmm);
 
 				/* convert to individual traces */
-  ImposeMasterTrace(aseq, ainfo.nseq, master, &addtr);
+  ImposeMasterTrace(msa->aseq, msa->nseq, master, &addtr);
 				/* add those traces to existing ones */
-  *tr = MergeTraceArrays(*tr, *nseq, addtr, ainfo.nseq);
+  *tr = MergeTraceArrays(*tr, *nseq, addtr, msa->nseq);
   
 				/* additional bookkeeping: add to dsq, sqinfo */
-  *rsq = ReallocOrDie((*rsq), sizeof(char *) * (*nseq + ainfo.nseq));
-  DealignAseqs(aseq, ainfo.nseq, &newrseq);
-  for (idx = *nseq; idx < *nseq + ainfo.nseq; idx++)
+  *rsq = ReallocOrDie((*rsq), sizeof(char *) * (*nseq + msa->nseq));
+  DealignAseqs(msa->aseq, msa->nseq, &newrseq);
+  for (idx = *nseq; idx < *nseq + msa->nseq; idx++)
     (*rsq)[idx] = newrseq[idx - (*nseq)];
   free(newrseq);
 
-  *dsq = ReallocOrDie((*dsq), sizeof(char *) * (*nseq + ainfo.nseq));
-  DigitizeAlignment(aseq, &ainfo, &newdsq);
-  for (idx = *nseq; idx < *nseq + ainfo.nseq; idx++)
+  *dsq = ReallocOrDie((*dsq), sizeof(char *) * (*nseq + msa->nseq));
+  DigitizeAlignment(msa, &newdsq);
+  for (idx = *nseq; idx < *nseq + msa->nseq; idx++)
     (*dsq)[idx] = newdsq[idx - (*nseq)];
   free(newdsq);
-				/* unnecessarily complex, but I can't be bothered... */
-  *sqinfo = ReallocOrDie((*sqinfo), sizeof(SQINFO) * (*nseq + ainfo.nseq));
-  for (idx = *nseq; idx < *nseq + ainfo.nseq; idx++)
-    SeqinfoCopy(&((*sqinfo)[idx]), &(ainfo.sqinfo[idx - (*nseq)]));
+			/* unnecessarily complex, but I can't be bothered... */
+  *sqinfo = ReallocOrDie((*sqinfo), sizeof(SQINFO) * (*nseq + msa->nseq));
+  for (idx = *nseq; idx < *nseq + msa->nseq; idx++)
+    SeqinfoCopy(&((*sqinfo)[idx]), &(newinfo[idx - (*nseq)]));
+  free(newinfo);
   
-  *nseq = *nseq + ainfo.nseq;
+  *nseq = *nseq + msa->nseq;
 
 				/* Cleanup */
   P7FreeTrace(master);
-  FreeAlignment(aseq, &ainfo);
+  MSAFree(msa);
 				/* Return */
   return;
 }
