@@ -1,6 +1,6 @@
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Copyright (C) 1992-1998 Washington University School of Medicine
+ * Copyright (C) 1992-2003 Washington University School of Medicine
  * All Rights Reserved
  * 
  *     This source code is distributed under the terms of the
@@ -12,22 +12,22 @@
  * SRE, Sun Mar  8 14:11:24 1998 [St. Louis]
  * 
  * main() for generating sequences from an HMM
- * RCS $Id: hmmemit.c,v 1.3 1998/12/09 00:44:26 eddy Exp $
+ * CVS $Id: hmmemit.c,v 1.8 2003/04/14 16:00:17 eddy Exp $
  */
 
+#include "config.h"		/* compile-time configuration constants */
+#include "squidconf.h"
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <time.h>
 
 #include "structs.h"		/* data structures, macros, #define's   */
-#include "config.h"		/* compile-time configuration constants */
 #include "funcs.h"		/* function declarations                */
 #include "globals.h"		/* alphabet global variables            */
 #include "squid.h"		/* general sequence analysis library    */
-
-#ifdef MEMDEBUG
-#include "dbmalloc.h"
-#endif
+#include "msa.h"		/* squid's multiple sequence i/o        */
 
 static char banner[] = "hmmemit - generate sequences from a profile HMM";
 
@@ -43,7 +43,7 @@ Available options are:\n\
 ";
 
 static char experts[] = "\
-   --seed <n> : set random number seed to <n>\n\
+   --seed <n>     : set random number seed to <n>\n\
 ";
 
 static struct opt_s OPTIONS[] = {
@@ -61,11 +61,12 @@ int
 main(int argc, char **argv) 
 {
   char            *hmmfile;	/* file to read HMMs from                  */
-  FILE            *fp;          /* output file handle                      */
   HMMFILE         *hmmfp;       /* opened hmmfile for reading              */
   struct plan7_s  *hmm;         /* HMM to generate from                    */
+  FILE            *fp;          /* output file handle                      */
   int              L;		/* length of a sequence                    */
   int              i;		/* counter over sequences                  */
+  int              nhmm;	/* counter over HMMs                       */
 
   char            *ofile;       /* output sequence file                    */
   int              nseq;	/* number of seqs to sample                */
@@ -78,12 +79,6 @@ main(int argc, char **argv)
   char *optarg;                 /* argument found by Getopt()               */
   int   optind;                 /* index in argv[]                          */
 
-#ifdef MEMDEBUG
-  unsigned long histid1, histid2, orig_size, current_size;
-  orig_size = malloc_inuse(&histid1);
-  fprintf(stderr, "[... memory debugging is ON ...]\n");
-#endif
-
   /*********************************************** 
    * Parse command line
    ***********************************************/
@@ -105,7 +100,7 @@ main(int argc, char **argv)
     else if (strcmp(optname, "--seed") == 0) seed         = atoi(optarg);
     else if (strcmp(optname, "-h") == 0) 
       {
-	Banner(stdout, banner);
+	HMMERBanner(stdout, banner);
 	puts(usage);
 	puts(experts);
 	exit(0);
@@ -125,144 +120,151 @@ main(int argc, char **argv)
 
   /*********************************************** 
    * Open HMM file (might be in HMMERDB or current directory).
-   * Read a single HMM from it.
+   * Open output file, if needed.
    ***********************************************/
 
   if ((hmmfp = HMMFileOpen(hmmfile, "HMMERDB")) == NULL)
     Die("Failed to open HMM file %s\n%s", hmmfile, usage);
-  if (!HMMFileRead(hmmfp, &hmm)) 
-    Die("Failed to read any HMMs from %s\n", hmmfile);
-  HMMFileClose(hmmfp);
-  if (hmm == NULL) 
-    Die("HMM file %s corrupt or in incorrect format? Parse failed", hmmfile);
-
-  /* Configure the HMM to shut off N,J,C emission: so we
-   * do a simple single pass through the model.
-   */
-  Plan7NakedConfig(hmm);
-  Plan7Renormalize(hmm);
-
-  /*********************************************** 
-   * Open the output file, or stdout
-   ***********************************************/ 
 
    if (ofile == NULL) fp = stdout;
    else {
      if ((fp = fopen(ofile, "w")) == NULL)
        Die("Failed to open output file %s for writing", ofile);
    }
- 
+
   /*********************************************** 
    * Show the options banner
    ***********************************************/
 
   if (! be_quiet) 
     {
-      Banner(stdout, banner);
+      HMMERBanner(stdout, banner);
       printf("HMM file:             %s\n", hmmfile);
       if (! do_consensus) {
 	printf("Number of seqs:       %d\n", nseq);
 	printf("Random seed:          %d\n", seed);
+      } else {
+	printf("Generating consensus sequence.\n");
       }
       printf("- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n\n");
     }
 
   /*********************************************** 
-   * Do the work.
-   * If we're generating an alignment, we have to collect
-   * all our traces, then output. If we're generating unaligned
-   * sequences, we can emit one at a time.
+   * For every HMM in the file, do some emission.
    ***********************************************/
 
-  if (do_consensus) 
-    {
-      char    *seq;
-      SQINFO   sqinfo;      /* info about sequence (name/desc)        */
+  nhmm = 0;
+  while (HMMFileRead(hmmfp, &hmm)) {
+    if (hmm == NULL) 
+      Die("HMM file %s corrupt or in incorrect format? Parse failed", hmmfile);
 
-      EmitConsensusSequence(hmm, &seq, NULL, &L, NULL);
-      strcpy(sqinfo.name, "consensus");
-      sqinfo.len = L;
-      sqinfo.flags = SQINFO_NAME | SQINFO_LEN;
+    /* Configure the HMM to shut off N,J,C emission: so we
+     * do a simple single pass through the model.
+     */
+    Plan7NakedConfig(hmm);
+    Plan7Renormalize(hmm);
 
-      WriteSeq(fp, kPearson, seq, &sqinfo);
-      free(seq);
-    }
-  else if (do_alignment)
-    {
-      struct p7trace_s **tr;        /* traces for aligned sequences            */
-      char           **dsq;         /* digitized sequences                     */
-      SQINFO          *sqinfo;      /* info about sequences (name/desc)        */
-      char           **aseq;        /* sequence alignment                      */
-      AINFO            ainfo;	    /* optional alignment info                 */
-      float           *wgt;
+    /*********************************************** 
+     * Do the work.
+     * If we're generating an alignment, we have to collect
+     * all our traces, then output. If we're generating unaligned
+     * sequences, we can emit one at a time.
+     ***********************************************/
 
-      dsq    = MallocOrDie(sizeof(char *)             * nseq);
-      tr     = MallocOrDie(sizeof(struct p7trace_s *) * nseq);
-      sqinfo = MallocOrDie(sizeof(SQINFO)             * nseq);
-      wgt    = MallocOrDie(sizeof(float)              * nseq);
-      FSet(wgt, nseq, 1.0);
+    if (do_consensus) 
+      {
+	char    *seq;
+	SQINFO   sqinfo;      /* info about sequence (name/desc)        */
+	
+	EmitConsensusSequence(hmm, &seq, NULL, &L, NULL);
+	strcpy(sqinfo.name, hmm->name);
+	strcpy(sqinfo.desc, "profile HMM generated consensus sequence [hmmemit]");
+	
+	sqinfo.len = L;
+	sqinfo.flags = SQINFO_NAME | SQINFO_DESC | SQINFO_LEN;
 
-      for (i = 0; i < nseq; i++)
-	{
-	  EmitSequence(hmm, &(dsq[i]), &L, &(tr[i]));
-	  sprintf(sqinfo[i].name, "seq%d", i+1);
-	  sqinfo[i].len   = L;
-	  sqinfo[i].flags = SQINFO_NAME | SQINFO_LEN;
-	}
+	WriteSeq(fp, SQFILE_FASTA, seq, &sqinfo);
+	free(seq);
+      }
+    else if (do_alignment)
+      {
+	struct p7trace_s **tr;        /* traces for aligned sequences            */
+	char           **dsq;         /* digitized sequences                     */
+	SQINFO          *sqinfo;      /* info about sequences (name/desc)        */
+	MSA             *msa;         /* alignment */
+	float           *wgt;
 
-      P7Traces2Alignment(dsq, sqinfo, wgt, nseq, hmm->M, tr, FALSE, 
-			 &aseq, &ainfo);
+	dsq    = MallocOrDie(sizeof(char *)             * nseq);
+	tr     = MallocOrDie(sizeof(struct p7trace_s *) * nseq);
+	sqinfo = MallocOrDie(sizeof(SQINFO)             * nseq);
+	wgt    = MallocOrDie(sizeof(float)              * nseq);
+	FSet(wgt, nseq, 1.0);
+
+	for (i = 0; i < nseq; i++)
+	  {
+	    EmitSequence(hmm, &(dsq[i]), &L, &(tr[i]));
+	    sprintf(sqinfo[i].name, "seq%d", i+1);
+	    sqinfo[i].len   = L;
+	    sqinfo[i].flags = SQINFO_NAME | SQINFO_LEN;
+	  }
+
+	msa = P7Traces2Alignment(dsq, sqinfo, wgt, nseq, hmm->M, tr, FALSE);
+	msa->name = sre_strdup(hmm->name, -1);
+	msa->desc = sre_strdup("Synthetic sequence alignment generated by hmmemit", -1);
 
 				/* Output the alignment */
-      WriteSELEX(fp, aseq, &ainfo, 50);
-      if (ofile != NULL && !be_quiet) printf("Alignment saved in file %s\n", ofile);
+	WriteStockholm(fp, msa);
 
-      /* Free memory
-       */
-      for (i = 0; i < nseq; i++) 
-	{
-	  P7FreeTrace(tr[i]);
-	  free(dsq[i]);
-	}
-      FreeAlignment(aseq, &ainfo);
-      free(sqinfo);
-      free(dsq);
-      free(wgt);
-      free(tr);
-    }
-  else				/* unaligned sequence output */
-    {
-      struct p7trace_s *tr;         /* generated trace                        */
-      char             *dsq;        /* digitized sequence                     */
-      char             *seq;        /* alphabetic sequence                    */
-      SQINFO            sqinfo;     /* info about sequence (name/len)         */
+	/* Free memory
+	 */
+	for (i = 0; i < nseq; i++) 
+	  {
+	    P7FreeTrace(tr[i]);
+	    free(dsq[i]);
+	  }
+	MSAFree(msa);
+	free(sqinfo);
+	free(dsq);
+	free(wgt);
+	free(tr);
+      }
+    else				/* unaligned sequence output */
+      {
+	struct p7trace_s *tr;         /* generated trace                        */
+	char             *dsq;        /* digitized sequence                     */
+	char             *seq;        /* alphabetic sequence                    */
+	SQINFO            sqinfo;     /* info about sequence (name/len)         */
 
-      for (i = 0; i < nseq; i++)
-	{
-	  EmitSequence(hmm, &dsq, &L, &tr);
-	  sprintf(sqinfo.name, "seq%d", i+1);
-	  sqinfo.len   = L;
-	  sqinfo.flags = SQINFO_NAME | SQINFO_LEN;
+	for (i = 0; i < nseq; i++)
+	  {
+	    EmitSequence(hmm, &dsq, &L, &tr);
+	    sprintf(sqinfo.name, "%s-%d", hmm->name, i+1);
+	    sqinfo.len   = L;
+	    sqinfo.flags = SQINFO_NAME | SQINFO_LEN;
 
-	  seq = DedigitizeSequence(dsq, L);
+	    seq = DedigitizeSequence(dsq, L);
 
-	  WriteSeq(fp, kPearson, seq, &sqinfo);
+	    WriteSeq(fp, SQFILE_FASTA, seq, &sqinfo);
 	  
-	  P7FreeTrace(tr);
-	  free(dsq);
-	  free(seq);
-	}
-    }
+	    P7FreeTrace(tr);
+	    free(dsq);
+	    free(seq);
+	  }
+      }
+    nhmm++;
+    FreePlan7(hmm);
+  }
 
-  if (ofile != NULL) fclose(fp);
-  FreePlan7(hmm);
+  /* We're done; clean up and exit.
+   */
+  if (nhmm == 0)
+    Die("Failed to read any HMMs from %s\n", hmmfile);
+  if (ofile != NULL) {
+    fclose(fp);
+    if (!be_quiet) printf("Output saved in file %s\n", ofile);
+  }
+  HMMFileClose(hmmfp);
   SqdClean();
-
-#ifdef MEMDEBUG
-  current_size = malloc_inuse(&histid2);
-  if (current_size != orig_size) malloc_list(2, histid1, histid2);
-  else fprintf(stderr, "[No memory leaks.]\n");
-#endif
   return 0;
 }
 
